"""
╭━━┳━━━┳━━━┳━━━┳━━━╮
╰┫┣┫╭━╮┣╮╭╮┣╮╭╮┃╭━╮┃
╱┃┃┃╰━━╮┃┃┃┃┃┃┃┃┃╱╰╯
╱┃┃╰━━╮┃┃┃┃┃┃┃┃┃┃╱╭╮
╭┫┣┫╰━╯┣╯╰╯┣╯╰╯┃╰━╯┃
╰━━┻━━━┻━━━┻━━━┻━━━╯
"""
__version__ = 1, 4, 5

import os
import re
import sys
import copy
import subprocess
import collections

from .isddc_dict import IsddcDict as idict

from . import isddc_time as itime
from . import isddc_os as ios
from . import isddc_log as ilog
from . import isddc_pool as ipool
from . import isddc_decorator as idec
from . import isddc_import as iimport
from . import isddc_crypto as icrypto
from . import isddc_baseclass as ibase
from . import isddc_overfunc as ioverfunc

iapi: 'isddc api'
issh: 'isddc ssh'
ikube: 'isddc kube'
ikafka: 'isddc kafka'
imongo: 'isddc mongo'
imysql: 'isddc mysql'
iinflux: 'isddc influx'
idbpool: 'isddc dbpool'


def __init__(config: dict):
    config = copy.deepcopy(config)

    for iname, iconf in config.items():
        try:
            imodule = iimport.module(f'{__name__}.isddc_{iname[1:]}')
        except ModuleNotFoundError:
            continue

        init: dict = iconf.pop('init', {})

        for name, params in iconf.items():
            for item in init.items():
                params.setdefault(*item)

            iobj = imodule.__init__(params)

            if not hasattr(imodule, '__default__'):
                setattr(imodule, '__default__', iobj)

            setattr(imodule, name, iobj)

        setattr(sys.modules[__name__], iname, imodule)


def cmd(command: str) -> str:
    """
    if exec command success:
        return result
    else:
        AssertionError
    """
    status, output = subprocess.getstatusoutput(command)
    assert status == 0, f'({command}): "{output}"'
    return output


def cmd_list_to_dict(
        cmd_list: str,
        split: str = None
) -> list:
    """Used to turn the command output result with a title
    into a dictionary, such as `kubectl get pod`."""
    result = [[value.strip() for value in line.split(split)]
              for line in cmd_list.splitlines()]
    keys = [key.lower() for key in result[0]]
    return [dict(zip(keys, values)) for values in result[1:]]


def prt(*args, color: int = 31, font: int = 0,
        end: str = '\n', sep: str = ' ', file: open = None):
    """
    Custom print font and color.

    Color:
        31-red 32-green 34-blue 37-Gray 38-black
    Font:
        0-normal 1-bold
    sep:
        string inserted between values, default a space.
    """
    info: str = sep.join(str(i) for i in args)
    format: str = f'\033[{font};{color};0m{info}\033[0m'
    print(format, end=end, file=file)


def dict_inter_process(data: dict, func=None) -> dict:
    """Dictionary internal processing
    Used to batch process values inside a dictionary.
    :param data: A dict.
    :param func: Your handler functions, ps:
        def func(name, value):
            if name == 'xxx':
                return process(value)
    """

    def one(data: dict):
        for name, value in data.items():
            data[name] = two(name, value)

        return data

    def two(name=None, value={}):
        value = func(name, value) or value

        if isinstance(value, dict):
            return one(value)

        if isinstance(value, collections.Iterable) \
                and not isinstance(value, str):
            return [two(value=v) for v in value]

        return value

    return one(data)


def in_container() -> bool:
    """Whether to run in the container."""
    if os.name == 'posix':
        if cmd('fdisk -l'):
            return False
        return True
    return False


def hump(name: str) -> str:
    if '_' in name:
        name: str = ''.join(_.capitalize() for _ in name.split('_'))

    if '-' in name:
        name: str = ''.join(_.capitalize() for _ in name.split('-'))

    return name[0].upper() + name[1:]


def underline(name: str) -> str:
    for _ in re.findall(r'([A-Z]+)[A-Z]', name):
        name = name.replace(_, _.capitalize())

    result: list = re.findall(r'[A-Z][a-z]+|[A-Z]\d', name)

    return '_'.join(_.lower() for _ in result)


def get_caller(level: int = 1) -> str:
    back = sys._getframe().f_back

    for _ in range(level):
        back = back.f_back

    return back.f_code.co_name


def mean(numbers: list) -> float:
    return sum(numbers) / len(numbers)
