import abc
import time
import functools
import isddc as i

from . import itime
from . import isddc_log as ilog


class BaseDecorator(metaclass=abc.ABCMeta):

    def __call__(self, func):
        @functools.wraps(func)
        def inner(*a, **kw):
            return self.core(func, *a, **kw)

        return inner

    @abc.abstractmethod
    def core(self, func, *a, **kw):
        ...

    @staticmethod
    def output(info: str, level: str = 'ERROR'):
        try:
            getattr(ilog.simple, level.lower())(info)
        except AttributeError:
            return print(f'[{itime.now.str}] [{level.upper()}] {info}')


class TryExcept(BaseDecorator):

    def __init__(
            self,
            exc_type: type = Exception,
            mark: str = None,
            ignore: bool = False,
            exc_return: type or tuple = None
    ):
        self.mark = mark
        self.ignore = ignore
        self.exc_type = exc_type
        self.exc_return = exc_return

    def core(self, func, *a, **kw):
        try:
            return func(*a, **kw)
        except self.exc_type as e:
            if not self.ignore:
                try:
                    self.exception_handler(func.__name__, e)
                except Exception as e:
                    self.output(f'[{itime.now.str}] TryExceptError: {e}')

        return self.exc_return

    def exception_handler(self, func_name: str, err: Exception):
        mark: str = self.mark or i.hump(func_name)
        exc_name: str = type(err).__name__
        self.output(f'{mark}.{exc_name}: {str(err)}')


class Retry(BaseDecorator):

    def __init__(
            self,
            mark: str = None,
            count: int = None,
            cycle: int = 10,
            retry_exc: type or tuple = Exception,
    ):
        self.mark = mark
        self.count = count
        self.cycle = cycle
        self.retry_exc = retry_exc

    def core(self, func, *a, **kw):
        count = 0

        while True:
            try:
                return func(*a, **kw)
            except self.retry_exc as e:
                try:
                    count += 1

                    sign: str = self.mark or i.hump(func.__name__)
                    exc_name: str = type(e).__name__

                    self.output(f'[count:{count}/{self.count or "N"}] {sign}.{exc_name}: {e}')

                except Exception as ee:
                    self.output(f'[{itime.now.str}] RetryError: {ee}')

                if self.count and count == self.count:
                    raise e

            time.sleep(self.cycle)


class WhileTrue(BaseDecorator):

    def __init__(
            self,
            cond=True,
            cycle: int = 0,
            before: bool = False
    ):
        self.cond = cond
        self.cycle = cycle
        self.before = before

    def core(self, func, *a, **kw):
        while self.cond:
            if self.before:
                time.sleep(self.cycle)

            ret = func(*a, **kw)

            if ret == '_break_':
                break

            if ret == '_continue_':
                continue

            if not self.before:
                time.sleep(self.cycle)


class RunTime(BaseDecorator):

    def __init__(self, mark: str = None):
        self.mark = mark

    def core(self, func, *a, **kw):
        start = time.time()
        func(*a, **kw)
        end = time.time()

        mark: str = self.mark or i.hump(func.__name__)
        exec_time: str = itime.second2time(end - start)

        self.output(f'{mark}: {exec_time}', level='INFO')


class TestFuncSpeed(BaseDecorator):

    def __init__(self, count: int, keep: int = 7):
        self.count = count
        self.keep = keep

    def core(self, func, *a, **kw):
        speeds = []

        for _ in range(self.count):
            start = time.time()
            func(*a, **kw)
            end = time.time()
            speeds.append(end - start)

        for action in min, max, i.mean, sum:
            result = round(action(speeds), self.keep)
            print(f'{action.__name__}: {result}')
